clear

# set variables
VERSION=`cat version.txt`
NAMESTRING="commit_linux_${VERSION}"
OUTNAMESTRING="${NAMESTRING}"
SHOULDCLEAN="0"
SHOULDTAR="0"
SHOULDDEB="0"
SHOULDRPM="0"
SHOULDDISABLEUPDATE="0"
SHOULDSOURCE="0"

# detect the CLI flags
while [ "$1" != "" ]; do
  if [ "$1" == "-c" ]; then
    SHOULDCLEAN="1"
  elif [ "$1" == "-t" ]; then
    SHOULDTAR="1"
  elif [ "$1" == "-d" ]; then
    SHOULDDEB="1"
  elif [ "$1" == "-r" ]; then
    SHOULDDEB="1"
    SHOULDRPM="1"
  elif [ "$1" == "-n" ]; then
    SHOULDDISABLEUPDATE="1"
  elif [ "$1" == "-s" ]; then
    SHOULDSOURCE="1"
  fi
  shift
done

# install correct SDKs
printf "Installing the correct Flatpak platform and SDK...\n"
flatpak install -y --noninteractive flathub org.freedesktop.Platform//21.08 org.freedesktop.Sdk//21.08

# cleanup
printf "Doing some cleanup...\n"
rm -rf dist
if [ "$SHOULDCLEAN" == "1" ]; then
  rm -rf build
fi

# make directories
printf "Making required directories...\n"
mkdir -p dist
mkdir -p build/flatpak/stage/commit/commit/assets/icons/typewriter
mkdir -p build/flatpak/stage/commit/commit/assets/sound
mkdir -p build/flatpak/stage/commit/commit/includes
mkdir -p build/flatpak/stage/commit/commit/ui
mkdir -p build/flatpak/build

# copy app files
printf "Copying files...\n"
cp commit.py *.md version.txt build/flatpak/stage/commit
cp commit/*.py build/flatpak/stage/commit/commit
cp -r commit/locale build/flatpak/stage/commit/commit/locale
cp commit/assets/*.png build/flatpak/stage/commit/commit/assets
cp -r commit/includes/*.py build/flatpak/stage/commit/commit/includes
cp -r commit/ui/*.py build/flatpak/stage/commit/commit/ui
cp -r commit/assets/icons/*.png build/flatpak/stage/commit/commit/assets/icons
cp -r commit/assets/sound/*.wav build/flatpak/stage/commit/commit/assets/sound

cp commit/assets/icons/appicon.svg build/flatpak/stage/
cp commit/assets/icons/docicon.svg build/flatpak/stage/
cp commit/assets/icons/appicon*.png build/flatpak/stage/
cp commit/assets/icons/docicon*.png build/flatpak/stage/

printf "Downloading Pandoc for inclusion in Flatpak...\n"
python3 << EOF
import pypandoc
import os
target_path = os.path.abspath("build/flatpak/stage/")
pypandoc.pandoc_download.download_pandoc(
    download_folder=target_path,
    targetfolder=target_path
)
EOF
mv ./build/flatpak/stage/copyright.pandoc ./build/flatpak/stage/pandoc-copyright.txt

# if we need to disable the update feature
if [ "$SHOULDDISABLEUPDATE" == "1" ]; then
  touch build/flatpak/stage/commit/noupdate
  OUTNAMESTRING="${NAMESTRING}_noupdate"
fi

printf "Generating the Python3 requirements for Flatpak...\n"
cat << EOF > build/flatpak/stage/python3-reqs.json
{
    "name": "python3-reqs",
    "buildsystem": "simple",
    "build-commands": [],
    "modules": [
        {
            "name": "python3-appdirs",
            "buildsystem": "simple",
            "build-commands": [
                "pip3 install --verbose --exists-action=i --no-index --prefix=/app --no-build-isolation appdirs-1.4.4-py2.py3-none-any.whl"
            ],
            "sources": [
                {
                    "type": "file",
                    "url": "https://files.pythonhosted.org/packages/3b/00/2344469e2084fb287c2e0b57b72910309874c3245463acd6cf5e3db69324/appdirs-1.4.4-py2.py3-none-any.whl",
                    "sha256": "a841dacd6b99318a741b166adb07e19ee71a274450e68237b4650ca1055ab128"
                }
            ]
        },
        {
            "name": "python3-lxml",
            "buildsystem": "simple",
            "build-commands": [
                "pip3 install --verbose --exists-action=i --no-index --prefix=/app --no-build-isolation ."
            ],
            "sources": [
                {
                    "type": "archive",
                    "url": "https://files.pythonhosted.org/packages/70/bb/7a2c7b4f8f434aa1ee801704bf08f1e53d7b5feba3d5313ab17003477808/lxml-4.9.1.tar.gz",
                    "sha256": "fe749b052bb7233fe5d072fcb549221a8cb1a16725c47c37e42b0b9cb3ff2c3f"
                }
            ]
        },
        {
            "name": "python3-markdown2",
            "buildsystem": "simple",
            "build-commands": [
                "pip3 install --verbose --exists-action=i --no-index --prefix=/app --no-build-isolation markdown2-2.4.6-py2.py3-none-any.whl"
            ],
            "sources": [
                {
                    "type": "file",
                    "url": "https://files.pythonhosted.org/packages/b2/58/68ae0307bb9f8b0e0fe79b1307ecbc22a31e77f30de89ca06ecadd99bbc7/markdown2-2.4.6-py2.py3-none-any.whl",
                    "sha256": "528f978beb3dbcf529a139b6c76f6ac6ecf8bb96c131beab751bb095b3873b58"
                }
            ]
        },
        {
            "name": "python3-pypandoc",
            "buildsystem": "simple",
            "build-commands": [
                "pip3 install --verbose --exists-action=i --no-index --prefix=/app --no-build-isolation pypandoc-1.10-py3-none-any.whl"
            ],
            "sources": [
                {
                    "type": "file",
                    "url": "https://files.pythonhosted.org/packages/12/0c/5c7f36bbc94bb6994071471a63f4e6f235bc02f4e959293bc23a9df54acb/pypandoc-1.10-py3-none-any.whl",
                    "sha256": "ff9658cda18865a822695349a7c707756ecc454b59b71f920b554579ec54aaa7"
                }
            ]
        },
        {
            "name": "python3-numpy",
            "buildsystem": "simple",
            "build-commands": [
                "pip3 install --verbose --exists-action=i --no-index --prefix=/app --no-build-isolation ."
            ],
            "sources": [
                {
                    "type": "archive",
                    "url": "https://files.pythonhosted.org/packages/42/38/775b43da55fa7473015eddc9a819571517d9a271a9f8134f68fb9be2f212/numpy-1.23.5.tar.gz",
                    "sha256": "1b1766d6f397c18153d40015ddfc79ddb715cabadc04d2d228d4e5a8bc4ded1a"
                }
            ]
        },
        {
            "name": "python3-pillow",
            "buildsystem": "simple",
            "build-commands": [
                "pip3 install --verbose --exists-action=i --no-index --prefix=/app --no-build-isolation ."
            ],
            "sources": [
                {
                    "type": "archive",
                    "url": "https://files.pythonhosted.org/packages/16/11/da8d395299ca166aa56d9436e26fe8440e5443471de16ccd9a1d06f5993a/Pillow-9.3.0.tar.gz",
                    "sha256": "c935a22a557a560108d780f9a0fc426dd7459940dc54faa49d83249c8d3e760f"
                }
            ]
        },
        {
            "name": "python3-wxpython",
            "buildsystem": "simple",
            "build-commands": [
                "pip3 install --verbose --exists-action=i --no-index --prefix=/app --no-build-isolation ."
            ],
            "sources": [
                {
                    "type": "archive",
                    "url": "https://files.pythonhosted.org/packages/b0/4d/80d65c37ee60a479d338d27a2895fb15bbba27a3e6bb5b6d72bb28246e99/wxPython-4.1.1.tar.gz",
                    "sha256": "00e5e3180ac7f2852f342ad341d57c44e7e4326de0b550b9a5c4a8361b6c3528"
                }
            ]
        }
    ]
}
EOF

printf "Creating Flatpak launch script...\n"
cat << EOF > build/flatpak/stage/commit.sh
#!/bin/sh
cd /app/bin/commit
python3 commit.py "\$1"
EOF
chmod +x build/flatpak/stage/commit.sh

printf "Writing the Flatpak desktop shortcut...\n"
cat << EOF > build/flatpak/stage/com.intersanity.commit.desktop
[Desktop Entry]
Version=1.0
Type=Application
Name=Commit
Icon=com.intersanity.commit
Exec=/app/bin/commit.sh "%f"
Comment=The advanced modern typewriter
Categories=Office;WordProcessor;
StartupNotify=true
Terminal=false
MimeType=text/com.intersanity.commit;
EOF

printf "Writing the MIME type file for the Flatpak...\n"
cat << EOF > build/flatpak/stage/com.intersanity.commit.xml
<?xml version="1.0" encoding="UTF-8"?>
<mime-info xmlns="http://www.freedesktop.org/standards/shared-mime-info">
  <mime-type type="text/com.intersanity.commit">
    <comment>Commit text document</comment>
    <glob pattern="*.ctxt"/>
    <icon name="com.intersanity.commit-text"/>
  </mime-type>
</mime-info>
EOF

printf "Creating the appdata XML file for the Flatpak...\n"
cat << EOF > build/flatpak/stage/com.intersanity.commit.appdata.xml
<?xml version="1.0" encoding="UTF-8"?>
<component>
  <id>com.intersanity.commit</id>
  <launchable type="desktop-id">com.intersanity.commit.desktop</launchable>
  <metadata_license>MIT</metadata_license>
  <project_license>MIT</project_license>
  <name>Commit</name>
  <summary>Commit, the advanced modern typewriter.</summary>
  <description>
    <p>Commit helps you focus on writing. It only lets you edit the current working paragraph. The rest is already committed. Keep writing.</p>
  </description>
  <icon type="local" width="64" height="64">/app/share/icons/hicolor/64x64/mimetypes/intersanity-commit.png</icon>
  <url type="homepage">https://www.intersanity.com/commit/</url>
  <url type="bugtracker">https://bugs.intersanity.com/</url>
  <url type="donation">https://www.intersanityc.com/commit/donate/</url>
  <releases>
    <release version="$VERSION"/>
  </releases>
  <provides>
    <mediatype>text/com.intersanity.commit</mediatype>
    <binary>commit.sh</binary>
  </provides>
  <developer_name>Intersanity Enterprises</developer_name>
</component>
EOF

printf "Creating Flatpak manifest file...\n"
cat << EOF > build/flatpak/stage/com.intersanity.commit.yml
app-id: com.intersanity.commit
runtime: org.freedesktop.Platform
runtime-version: '21.08'
sdk: org.freedesktop.Sdk
command: commit.sh
modules:
  - python3-reqs.json
  - name: commit
    buildsystem: simple
    build-commands:
      - install -D commit.sh /app/bin/commit.sh
      - install -D appicon.svg /app/share/icons/hicolor/scalable/apps/com.intersanity.commit.svg
      - install -D appicon16.png /app/share/icons/hicolor/16x16/apps/com.intersanity.commit.png
      - install -D appicon24.png /app/share/icons/hicolor/24x24/apps/com.intersanity.commit.png
      - install -D appicon32.png /app/share/icons/hicolor/32x32/apps/com.intersanity.commit.png
      - install -D appicon48.png /app/share/icons/hicolor/48x48/apps/com.intersanity.commit.png
      - install -D appicon96.png /app/share/icons/hicolor/96x96/apps/com.intersanity.commit.png
      - install -D appicon128.png /app/share/icons/hicolor/128x128/apps/com.intersanity.commit.png
      - install -D appicon256.png /app/share/icons/hicolor/256x256/apps/com.intersanity.commit.png
      - install -D appicon512.png /app/share/icons/hicolor/512x512/apps/com.intersanity.commit.png
      - install -D docicon.svg /app/share/icons/hicolor/scalable/mimetypes/com.intersanity.commit-text.svg
      - install -D docicon16.png /app/share/icons/hicolor/16x16/mimetypes/com.intersanity.commit-text.png
      - install -D docicon24.png /app/share/icons/hicolor/24x24/mimetypes/com.intersanity.commit-text.png
      - install -D docicon32.png /app/share/icons/hicolor/32x32/mimetypes/com.intersanity.commit-text.png
      - install -D docicon48.png /app/share/icons/hicolor/48x48/mimetypes/com.intersanity.commit-text.png
      - install -D docicon64.png /app/share/icons/hicolor/64x64/mimetypes/com.intersanity.commit-text.png
      - install -D docicon96.png /app/share/icons/hicolor/96x96/mimetypes/com.intersanity.commit-text.png
      - install -D docicon128.png /app/share/icons/hicolor/128x128/mimetypes/com.intersanity.commit-text.png
      - install -D docicon256.png /app/share/icons/hicolor/256x256/mimetypes/com.intersanity.commit-text.png
      - install -D docicon512.png /app/share/icons/hicolor/512x512/mimetypes/com.intersanity.commit-text.png
      - install -D com.intersanity.commit.desktop /app/share/applications/com.intersanity.commit.desktop
      - install -D com.intersanity.commit.xml /app/share/mime/packages/com.intersanity.commit.xml
      - install -D com.intersanity.commit.appdata.xml /app/share/metainfo/com.intersanity.commit.appdata.xml
      - install -D pandoc /app/bin/pandoc
      - install -D pandoc-copyright.txt /app/bin/pandoc-copyright.txt
      - cp -R ../commit /app/bin/commit
    sources:
      - type: file
        path: commit.sh
      - type: file
        path: appicon.svg
      - type: file
        path: appicon16.png
      - type: file
        path: appicon24.png
      - type: file
        path: appicon32.png
      - type: file
        path: appicon48.png
      - type: file
        path: appicon64.png
      - type: file
        path: appicon96.png
      - type: file
        path: appicon128.png
      - type: file
        path: appicon256.png
      - type: file
        path: appicon512.png
      - type: file
        path: docicon.svg
      - type: file
        path: docicon16.png
      - type: file
        path: docicon24.png
      - type: file
        path: docicon32.png
      - type: file
        path: docicon48.png
      - type: file
        path: docicon64.png
      - type: file
        path: docicon96.png
      - type: file
        path: docicon128.png
      - type: file
        path: docicon256.png
      - type: file
        path: docicon512.png
      - type: file
        path: com.intersanity.commit.desktop
      - type: file
        path: com.intersanity.commit.xml
      - type: file
        path: com.intersanity.commit.appdata.xml
      - type: file
        path: pandoc
      - type: file
        path: pandoc-copyright.txt
      - type: dir
        path: commit
finish-args:
  - --share=ipc
  - --socket=x11
  - --socket=wayland
  - --share=network
  - --filesystem=xdg-documents
EOF

printf "Building the Flatpak...\n"
cd build/flatpak/stage/
flatpak-builder --repo=../repo --force-clean ../build com.intersanity.commit.yml
cd ../../../
flatpak build-bundle build/flatpak/repo dist/$OUTNAMESTRING.flatpak com.intersanity.commit

if [ "$SHOULDDEB" == "1" ]; then
  printf "Organizing files for the DEB...\n"

  mkdir -p build/$NAMESTRING/usr/sbin
  mkdir -p build/$NAMESTRING/usr/share/applications
  mkdir -p build/$NAMESTRING/usr/share/mime/packages
  mkdir -p build/$NAMESTRING/usr/share/icons/hicolor/scalable/mimetypes/
  mkdir -p build/$NAMESTRING/usr/share/icons/hicolor/scalable/apps/
  mkdir -p build/$NAMESTRING/usr/share/icons/hicolor/64x64/mimetypes/
  mkdir -p build/$NAMESTRING/usr/share/icons/hicolor/64x64/apps/
  mkdir -p build/$NAMESTRING/DEBIAN

  cp build/flatpak/stage/appicon.svg build/$NAMESTRING/usr/share/icons/hicolor/scalable/apps/com.intersanity.commit.svg
  cp build/flatpak/stage/appicon64.png build/$NAMESTRING/usr/share/icons/hicolor/64x64/apps/com.intersanity.commit.png
  cp build/flatpak/stage/docicon.svg build/$NAMESTRING/usr/share/icons/hicolor/scalable/mimetypes/com.intersanity.commit-text.svg
  cp build/flatpak/stage/docicon64.png build/$NAMESTRING/usr/share/icons/hicolor/64x64/mimetypes/com.intersanity.commit-text.png
  cp build/flatpak/stage/com.intersanity.commit.xml build/$NAMESTRING/usr/share/mime/packages/com.intersanity.commit.xml

  # copy everything else from flatpak stage
  cp -r build/flatpak/stage/commit build/$NAMESTRING/usr/share/commit/

  printf "Writing the DEB start script...\n"
  cat << EOF > build/$NAMESTRING/usr/sbin/commit.sh
#!/bin/sh
cd /usr/share/commit.sh
python3 commit.py "\$1"
EOF
  chmod +x build/$NAMESTRING/usr/sbin/commit

  printf "Writing the DEB desktop shortcut...\n"
  cat << EOF > build/$NAMESTRING/usr/share/applications/com.intersanity.commit.desktop
[Desktop Entry]
Version=1.0
Type=Application
Name=Commit
Icon=com.intersanity.commit
Exec=/usr/sbin/commit.sh "%f"
Comment=The advanced modern typewriter
Categories=Office;WordProcessor;
StartupNotify=true
Terminal=false
MimeType=text/com.intersanity.commit;
EOF

  printf "Writing the DEB control file...\n"
  cat << EOF > build/$NAMESTRING/DEBIAN/control
Package: commit
Version: $VERSION-1
Section: editors
Priority: optional
Architecture: all
Depends: osspd, pandoc, python3, python3-appdirs, python3-markdown2, python3-pypandoc, python3-wxgtk4.0
Installed-Size: 440
Maintainer: Intersanity Enterprises <admin@intersanity.com>
Description: The advanced modern typewriter
 Commit helps you focus on writing. It only lets you edit the current working paragraph. The rest is already committed. Keep writing.
Homepage: https://www.intersanity.com/commit/
EOF

  printf "Building the DEB...\n"
  dpkg-deb --build -Z gzip --root-owner-group build/$NAMESTRING dist
  mv ./dist/*.deb ./dist/$OUTNAMESTRING.deb
fi

# create RPM file
if [ "$SHOULDRPM" == "1" ]; then
  printf "Converting our DEB to an RPM...\n"
  cd dist || exit
  alien -r $OUTNAMESTRING.deb
  cd ..
  mv ./dist/*.rpm ./dist/$OUTNAMESTRING.rpm
fi

# create the TAR.GZ manual distribution
if [ "$SHOULDTAR" == "1" ]; then
  printf "Creating a TAR.GZ for manual distribution...\n"
  mkdir build/tarball
  cp -R ./build/flatpak/stage/commit build/tarball/commit
  cp ./build/flatpak/stage/pandoc ./build/flatpak/stage/pandoc-copyright.txt build/tarball/commit/
  cat << EOF > build/tarball/commit/commit.sh
#!/bin/sh
python3 commit.py "$1"
EOF
  chmod +x build/tarball/commit/commit.sh
  tar -C ./build/tarball -czvf ./dist/$OUTNAMESTRING.tar.gz commit
fi

# create a source tarball
if [ "$SHOULDSOURCE" == "1" ]; then
  printf "Creating the source tarball...\n"
  tar --exclude='.git' --exclude='.idea' --exclude='__pycache__' --exclude='./build' --exclude='./dist' --exclude='./venv' -czvf ./dist/commit_${VERSION}_source.tar.gz ./
fi

# done
printf "Our work here is done.\n"
