import pathlib          # Python Pathlib - https://docs.python.org/3/library/pathlib.html
import sys
from setuptools import setup

version = pathlib.Path("./version.txt").read_text()

app = ['commit.py']
data_files = [
    (
        "", [
            "version.txt",
            "CREDITS.md",
            "LICENSE.md"
        ]
    ),
    (
        "commit/assets", [
            "commit/assets/commit-splash.png"
        ]
    ),
    (
        "commit/assets/icons", [
            "commit/assets/icons/appicon16.png",
            "commit/assets/icons/appicon24.png",
            "commit/assets/icons/appicon32.png",
            "commit/assets/icons/appicon48.png",
            "commit/assets/icons/appicon64.png",
            "commit/assets/icons/appicon96.png",
            "commit/assets/icons/appicon128.png",
            "commit/assets/icons/appicon256.png",
            "commit/assets/icons/appicon512.png",
            "commit/assets/icons/appicon1024.png"
        ]
    ),
    (
        "commit/assets/sound", [
            "commit/assets/sound/commit.wav",
            "commit/assets/sound/pull.wav"
        ]
    )
]

# conditionals
if sys.platform == "darwin":
    import py2app
    data_files.append(
        (
            "commit/assets/toolbar/mac", [
                "commit/assets/toolbar/mac/chevron-down-24.png",
                "commit/assets/toolbar/mac/chevron-up-24.png",
                "commit/assets/toolbar/mac/escape-24.png",
                "commit/assets/toolbar/mac/full-screen-24.png",
                "commit/assets/toolbar/mac/full-screen-back-24.png",
                "commit/assets/toolbar/mac/gear-24.png"
            ]
        )
    )
    data_files.append(
        (
            "commit/assets/icons", [
                "commit/assets/icons/docicon.icns"
            ]
        )
    )
    # setup values
    setup(
        app=app,
        name="Commit",
        version=version,
        description="the Advanced Modern Typewriter",
        author="Intersanity Enterprises and contributors",
        data_files=data_files,
        options={'py2app': {
            "plist": {
                "CFBundleIdentifier": "com.intersanity.commit",
                "CFBundleDocumentTypes": [
                    {
                        "CFBundleTypeExtensions": ["ctxt"],
                        "CFBundleTypeName": "Commit text file",
                        "CFBundleTypeRole": "Editor",
                        "CFBundleTypeIconFile": "commit/assets/icons/docicon.icns",
                        "LSHandlerRank": "Owner"
                    }
                ]
            }
        }},
        setup_requires=['py2app']
    )

elif sys.platform == "win32":
    import py2exe
    data_files.append(
        (
            "commit/assets/toolbar/windows/", [
                "commit/assets/toolbar/windows/commit-24.png",
                "commit/assets/toolbar/windows/fullscreen-24.png",
                "commit/assets/toolbar/windows/fullscreen-back-24.png",
                "commit/assets/toolbar/windows/prefs-24.png",
                "commit/assets/toolbar/windows/pull-24.png",
                "commit/assets/toolbar/windows/quit-24.png"
            ]
        )
    )
    setup(
        name="Commit",
        version=version,
        description="the Advanced Modern Typewriter",
        author="Intersanity Enterprises and contributors",
        windows=[
            {
                "script": app[0],
                "icon_resources": [
                    (0, "commit/assets/icons/appicon.ico"),
                    (1, "commit/assets/icons/docicon.ico")
                ]
            }
        ],
        data_files=data_files,
        options={
            "py2exe": {
                "includes": ["wx._xml", 'lxml._elementpath'],
                "dist_dir": "dist\commit"
            }
        }
    )

else:
    setup(
        app=app,
        name="Commit",
        version=version,
        description="the Advanced Modern Typewriter",
        author="Intersanity Enterprises and contributors",
        data_files=data_files
    )
